<?php
/* --------------------------------------------------------------
   AutoUpdaterController.inc.php 2019-01-04
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2019 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

MainFactory::load_class('AdminHttpViewController');

/**
 * Class AutoUpdaterController
 */
class AutoUpdaterController extends AdminHttpViewController
{
	/**
	 * @var \AutoUpdater
	 */
	protected $autoUpdater;
	
	/**
	 * @var \DataCache
	 */
	protected $dataCache;
	
	/**
	 * @var \LanguageTextManager
	 */
	private $textPhrases;
	
	/**
	 * @var \UserConfigurationService
	 */
	protected $userConfigService;
	
	/**
	 * @var string
	 */
	protected $smartyTemplateDirectory;
	
	/**
	 * @var string
	 */
	protected $templateDirectory;
	
	/**
	 * @var string
	 */
	protected $cssDirectory;
	
	/**
	 * @var string
	 */
	protected $jsDirectory;
	
	
	/**
	 * Initial method for this controller.
	 */
	public function init()
	{
		$db         = StaticGXCoreLoader::getDatabaseQueryBuilder();
		$dataCache  = DataCache::get_instance();
		$logControl = LogControl::get_instance();
		
		$autoUpdaterFactory = new AutoUpdaterFactory($db, $dataCache, $logControl);
		
		$this->autoUpdater       = $autoUpdaterFactory->createAutoUpdater();
		$this->textPhrases       = MainFactory::create('LanguageTextManager', 'auto_updater',
		                                               $_SESSION['languages_id']);
		$this->dataCache         = DataCache::get_instance();
		$this->userConfigService = StaticGXCoreLoader::getService('UserConfiguration');
		
		$this->smartyTemplateDirectory = 'auto_updater/';
		$this->templateDirectory       = DIR_FS_CATALOG . 'admin/html/content/auto_updater/';
		$this->cssDirectory            = '../admin/html/assets/styles/modules/auto_updater/';
		$this->jsDirectory             = '../admin/html/assets/javascript/modules/auto_updater/';
	}
	
	
	/**
	 * Shows the available updates.
	 *
	 * @return \AdminLayoutHttpControllerResponse
	 */
	public function actionDefault()
	{
		if(gm_get_conf('AUTO_UPDATER_ACCEPT_DATA_PROCESSING') !== 'true')
		{
			return $this->actionDataProcessing();
		}
		
		if(!isset($_COOKIE['auto_updater_admin_check']) || $_COOKIE['auto_updater_admin_check'] !== 'admin_logged_in')
		{
			setcookie('auto_updater_admin_check', 'admin_logged_in', time() + 5 * 60);
		}
		
		$this->dataCache->clear_cache('auto-updater-update-status');
		$this->userConfigService->setUserConfiguration(new IdType($_SESSION['customer_id']), 'auto-updater-show-info',
		                                               'v2.0.2');
		
		$currentSection = 'overview';
		$title          = $this->textPhrases->get_text('module_title');
		$template       = 'overview.html';
		$data           = [
			'templateDir' => $this->smartyTemplateDirectory,
			'showInfo'    => $this->userConfigService->getUserConfiguration(new IdType($_SESSION['customer_id']),
			                                                                'auto-updater-show-info') !== 'v2.0.2',
		];
		try
		{
			$data['availableUpdates']   = $this->autoUpdater->getAvailableUpdates();
			$data['uninstalledUpdates'] = $this->autoUpdater->areUninstalledUpdatesAvailable();
		}
		catch(AutoUpdaterException $e)
		{
			$data['availableUpdates']   = [];
			$data['uninstalledUpdates'] = false;
		}
		
		return $this->_returnHttpResponse($title, $template, $data, $currentSection);
	}
	
	
	/**
	 * Shows the available backups.
	 *
	 * @return \AdminLayoutHttpControllerResponse
	 */
	public function actionBackups()
	{
		$GLOBALS['messageStack']->add($this->textPhrases->get_text('text_backup_notice'), 'error', 'error');
		
		$currentSection = 'backups';
		$title          = $this->textPhrases->get_text('module_title');
		$template       = 'backups.html';
		$data           = [
			'templateDir'      => $this->smartyTemplateDirectory,
			'availableBackups' => $this->autoUpdater->getAvailableBackups(),
		];
		
		return $this->_returnHttpResponse($title, $template, $data, $currentSection);
	}
	
	
	/**
	 * Displays the data processing information.
	 *
	 * @return \AdminLayoutHttpControllerResponse
	 */
	public function actionDataProcessing()
	{
		$currentSection = 'data_processing';
		$title          = $this->textPhrases->get_text('module_title');
		$template       = 'data_processing.html';
		$data           = [];
		
		return $this->_returnHttpResponse($title, $template, $data, $currentSection);
	}
	
	
	/**
	 * Accepting the data processing.
	 *
	 * @return \RedirectHttpControllerResponse
	 */
	public function actionAcceptDataProcessing()
	{
		gm_set_conf('AUTO_UPDATER_ACCEPT_DATA_PROCESSING', 'true');
		
		return MainFactory::create('RedirectHttpControllerResponse', 'admin.php?do=AutoUpdater');
	}
	
	
	/**
	 * Dissenting the data processing.
	 *
	 * @return \RedirectHttpControllerResponse
	 */
	public function actionDissentDataProcessing()
	{
		gm_set_conf('AUTO_UPDATER_ACCEPT_DATA_PROCESSING', 'false');
		
		return MainFactory::create('RedirectHttpControllerResponse', 'admin.php?do=AutoUpdater');
	}
	
	
	/**************************************************************************************************************
	 * Controller helper methods
	 **************************************************************************************************************/
	
	/**
	 * Creates and returns an AdminLayoutHttpControllerResponse.
	 *
	 * @param string $title
	 * @param string $template
	 * @param array  $templateData
	 * @param string $currentSection
	 *
	 * @return \AdminLayoutHttpControllerResponse
	 */
	protected function _returnHttpResponse($title = '',
	                                       $template = 'overview.html',
	                                       $templateData = [],
	                                       $currentSection = '')
	{
		if($title === '')
		{
			$title = $this->textPhrases->get_text('module_title');
		}
		
		return MainFactory::create('AdminLayoutHttpControllerResponse', new NonEmptyStringType($title),
		                           new ExistingFile(new NonEmptyStringType($this->templateDirectory . $template)),
		                           new KeyValueCollection($templateData), $this->_getAssets(),
		                           $this->_createContentNavigation($currentSection));
	}
	
	
	/**
	 * Returns the assets.
	 *
	 * @return \AssetCollection
	 */
	protected function _getAssets()
	{
		$assets = MainFactory::create('AssetCollection');
		$assets->add(MainFactory::create('Asset', $this->cssDirectory . 'auto_updater_v2.1.css'));
		$assets->add(MainFactory::create('Asset', $this->jsDirectory . 'auto_updater_update_processing_v2.1.js'));
		$assets->add(MainFactory::create('Asset', $this->jsDirectory . 'auto_updater_backups_overview_v2.1.js'));
		$assets->add(MainFactory::create('Asset', $this->jsDirectory . 'auto_updater_db_backup_creation_v2.1.js'));
		$assets->add(MainFactory::create('Asset', 'auto_updater.lang.inc.php'));
		$assets->add(MainFactory::create('Asset', 'admin_buttons.lang.inc.php'));
		
		return $assets;
	}
	
	
	/**
	 * Returns the content navigation.
	 *
	 * @param string $currentSection Defines the current navigation item.
	 *
	 * @return \ContentNavigationCollection
	 */
	protected function _createContentNavigation($currentSection = '')
	{
		$subNavigationItems = [
			'overview'        => [
				'title' => new StringType($this->textPhrases->get_text('heading_overview')),
				'url'   => new StringType('admin.php?do=AutoUpdater'),
			],
			'backups'         => [
				'title' => new StringType($this->textPhrases->get_text('heading_backups')),
				'url'   => new StringType('admin.php?do=AutoUpdater/backups'),
			],
			'data_processing' => [
				'title' => new StringType($this->textPhrases->get_text('heading_data_processing')),
				'url'   => new StringType('admin.php?do=AutoUpdater/dataProcessing'),
			],
		];
		
		$contentNavigation = MainFactory::create('ContentNavigationCollection', []);
		
		foreach($subNavigationItems as $itemName => $subNavigationItem)
		{
			$contentNavigation->add($subNavigationItem['title'], $subNavigationItem['url'],
			                        new BoolType($currentSection === $itemName));
		}
		
		return $contentNavigation;
	}
}